public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
From: Andreas Hindborg <a.hindborg@kernel.org>
To: "Tamir Duberstein" <tamird@gmail.com>,
	"Miguel Ojeda" <ojeda@kernel.org>,
	"Alex Gaynor" <alex.gaynor@gmail.com>,
	"Boqun Feng" <boqun.feng@gmail.com>,
	"Gary Guo" <gary@garyguo.net>,
	"Björn Roy Baron" <bjorn3_gh@protonmail.com>,
	"Benno Lossin" <lossin@kernel.org>,
	"Alice Ryhl" <aliceryhl@google.com>,
	"Trevor Gross" <tmgross@umich.edu>,
	"Danilo Krummrich" <dakr@kernel.org>
Cc: Daniel Gomez <da.gomez@kernel.org>,
	rust-for-linux@vger.kernel.org,  linux-kernel@vger.kernel.org,
	Andreas Hindborg <a.hindborg@kernel.org>
Subject: [PATCH 05/10] rust: xarray: use `xas_load` instead of `xa_load` in `Guard::load`
Date: Wed, 03 Dec 2025 23:26:35 +0100	[thread overview]
Message-ID: <20251203-xarray-entry-send-v1-5-9e5ffd5e3cf0@kernel.org> (raw)
In-Reply-To: <20251203-xarray-entry-send-v1-0-9e5ffd5e3cf0@kernel.org>

Replace the call to `xa_load` with `xas_load` in `Guard::load`. The
`xa_load` function takes the XArray lock internally, which would cause
a double lock since the `Guard` already holds the lock. The `xas_load`
function operates on XArray state and assumes the lock is already held,
which is the correct API to use when holding a `Guard`.

This change also removes the `#[expect(dead_code)]` annotation from
`XArrayState` and its constructor, as they are now in use.

Signed-off-by: Andreas Hindborg <a.hindborg@kernel.org>
---
 rust/kernel/xarray.rs | 8 ++++----
 1 file changed, 4 insertions(+), 4 deletions(-)

diff --git a/rust/kernel/xarray.rs b/rust/kernel/xarray.rs
index f6e610b059625..0f69a523b72bf 100644
--- a/rust/kernel/xarray.rs
+++ b/rust/kernel/xarray.rs
@@ -215,8 +215,10 @@ fn load<F, U>(&self, index: usize, f: F) -> Option<U>
     where
         F: FnOnce(NonNull<c_void>) -> U,
     {
-        // SAFETY: `self.xa.xa` is always valid by the type invariant.
-        let ptr = unsafe { bindings::xa_load(self.xa.xa.get(), index) };
+        let mut state = XArrayState::new(self, index);
+        // SAFETY: `state.state` is always valid by the type invariant of
+        // `XArrayState`.
+        let ptr = unsafe { bindings::xas_load(&raw mut state.state) };
         let ptr = NonNull::new(ptr.cast())?;
         Some(f(ptr))
     }
@@ -327,7 +329,6 @@ pub fn store(
 /// # Invariants
 ///
 /// - `state` is always a valid `bindings::xa_state`.
-#[expect(dead_code)]
 pub(crate) struct XArrayState<'a, 'b, T: ForeignOwnable> {
     /// Holds a reference to the lock guard to ensure the lock is not dropped
     /// while `Self` is live.
@@ -336,7 +337,6 @@ pub(crate) struct XArrayState<'a, 'b, T: ForeignOwnable> {
 }
 
 impl<'a, 'b, T: ForeignOwnable> XArrayState<'a, 'b, T> {
-    #[expect(dead_code)]
     fn new(access: &'b Guard<'a, T>, index: usize) -> Self {
         let ptr = access.xa.xa.get();
         // INVARIANT: We initialize `self.state` to a valid value below.

-- 
2.51.2



  parent reply	other threads:[~2025-12-03 22:27 UTC|newest]

Thread overview: 54+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-12-03 22:26 [PATCH 00/10] rust: xarray: add entry API with preloading Andreas Hindborg
2025-12-03 22:26 ` [PATCH 01/10] rust: xarray: minor formatting fixes Andreas Hindborg
2025-12-29 15:56   ` Tamir Duberstein
2025-12-29 15:58     ` Tamir Duberstein
2026-01-21 15:45   ` Gary Guo
2025-12-03 22:26 ` [PATCH 02/10] rust: xarray: add debug format for `StoreError` Andreas Hindborg
2025-12-29 15:58   ` Tamir Duberstein
2026-01-21 15:46   ` Gary Guo
2025-12-03 22:26 ` [PATCH 03/10] rust: xarray: add `contains_index` method Andreas Hindborg
2025-12-29 15:59   ` Tamir Duberstein
2026-01-07 18:34     ` Andreas Hindborg
2026-01-07 18:38       ` Tamir Duberstein
2026-01-08  9:29         ` Andreas Hindborg
2026-01-08 11:05           ` Tamir Duberstein
2026-01-09 10:38             ` Andreas Hindborg
2026-01-09 15:59               ` Tamir Duberstein
2026-01-09 17:04                 ` Gary Guo
2025-12-03 22:26 ` [PATCH 04/10] rust: xarray: add `XArrayState` Andreas Hindborg
2025-12-29 16:04   ` Tamir Duberstein
2026-01-07 18:48     ` Andreas Hindborg
2026-01-08 14:22       ` Gary Guo
2025-12-03 22:26 ` Andreas Hindborg [this message]
2025-12-29 16:06   ` [PATCH 05/10] rust: xarray: use `xas_load` instead of `xa_load` in `Guard::load` Tamir Duberstein
2026-01-07 19:19     ` Andreas Hindborg
2026-01-07 20:06       ` Tamir Duberstein
2026-01-08  9:30         ` Andreas Hindborg
2025-12-03 22:26 ` [PATCH 06/10] rust: xarray: simplify `Guard::load` Andreas Hindborg
2025-12-29 16:07   ` Tamir Duberstein
2026-01-07 19:25     ` Andreas Hindborg
2026-01-07 20:08       ` Tamir Duberstein
2026-01-08  9:38         ` Andreas Hindborg
2026-01-08 11:07           ` Tamir Duberstein
2026-01-09 16:57             ` Gary Guo
2025-12-03 22:26 ` [PATCH 07/10] rust: xarray: add `find_next` and `find_next_mut` Andreas Hindborg
2025-12-29 16:11   ` Tamir Duberstein
2026-01-07 19:29     ` Andreas Hindborg
2025-12-03 22:26 ` [PATCH 08/10] rust: xarray: add entry API Andreas Hindborg
2025-12-29 16:25   ` Tamir Duberstein
2026-01-08  9:25     ` Andreas Hindborg
2026-01-08 11:03       ` Tamir Duberstein
2026-01-09 10:36         ` Andreas Hindborg
2026-01-09 16:00           ` Tamir Duberstein
2025-12-03 22:26 ` [PATCH 09/10] rust: xarray: add preload API Andreas Hindborg
2026-01-21 15:56   ` Gary Guo
2026-01-21 19:09     ` Andreas Hindborg
2025-12-03 22:26 ` [PATCH 10/10] rust: xarray: fix false positive lockdep warnings Andreas Hindborg
2025-12-29 16:29   ` Tamir Duberstein
2026-02-06 14:22     ` Andreas Hindborg
2026-01-21 15:59   ` Gary Guo
2026-01-21 19:01     ` Andreas Hindborg
2026-01-21 19:10       ` Gary Guo
2026-01-21 20:17         ` Andreas Hindborg
2026-01-22  9:20   ` Alice Ryhl
2026-02-06 14:21     ` Andreas Hindborg

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20251203-xarray-entry-send-v1-5-9e5ffd5e3cf0@kernel.org \
    --to=a.hindborg@kernel.org \
    --cc=alex.gaynor@gmail.com \
    --cc=aliceryhl@google.com \
    --cc=bjorn3_gh@protonmail.com \
    --cc=boqun.feng@gmail.com \
    --cc=da.gomez@kernel.org \
    --cc=dakr@kernel.org \
    --cc=gary@garyguo.net \
    --cc=linux-kernel@vger.kernel.org \
    --cc=lossin@kernel.org \
    --cc=ojeda@kernel.org \
    --cc=rust-for-linux@vger.kernel.org \
    --cc=tamird@gmail.com \
    --cc=tmgross@umich.edu \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox